/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.tools;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.NodeTypeDescriptor;
import com.unitesk.requality.eclipse.natures.ReqCatalogue;

public class TreesTracker {
    public enum TreeEvent {
        TREE_OPEN, TREE_CLOSE
    }

    private int inactivityCounter;

    private Map<IProject, TreeDB> treeByProject = new HashMap<IProject, TreeDB>();
    private Map<TreeDB, IProject> projectByTree = new HashMap<TreeDB, IProject>();
    private static TreesTracker tracker;
    private List<ITreeStateListener> listeners = new ArrayList<ITreeStateListener>();

    public boolean addListener(ITreeStateListener l) {
        return listeners.add(l);
    }

    public boolean removeListener(ITreeStateListener l) {
        return listeners.remove(l);
    }

    private TreesTracker() {
    }

    public static TreesTracker getTracker() {
        if (tracker == null)
            tracker = new TreesTracker();
        return tracker;
    }

    public synchronized void notifyListeners(TreeDB db, TreeEvent type) {
        if (isInactive()) {
            return;
        }
        for (ITreeStateListener listener : listeners) {
            switch (type) {
            case TREE_CLOSE:
                listener.close(db);
                break;
            case TREE_OPEN:
                listener.open(db);
                break;
            }
        }
    }

    public IProject getProject(TreeDB cat) {
        return projectByTree.get(cat);
    }

    public TreeDB getTree(IProject proj) {
        return treeByProject.get(proj);
    }

    public boolean trackProject(IProject project) {
        try {
            if (project.isOpen() && project.isNatureEnabled(ReqCatalogue.ID)
                    && !isTracking(project)) {
                TreeDB db = null;
                db = createTreeDB(project);
                Map<String, NodeTypeDescriptor> nts = Activator.getAdditionalTypes();
                for(String name: Activator.getAdditionalTypes().keySet()) {
                    db.addNodeType(name, nts.get(name).getClazz());
                }
                db.addListener(new DefaultTreeChangeListener(){
                    @Override
                    public void deleted(TreeNode node) {
                        // TODO: is it right place for this?
                        if(Activator.getDefault().getActiveNode(node.getType())==null) {
                            return;
                        }
                        if(Activator.getDefault().getActiveNode(node.getType()).equals(node)) {
                            Activator.getDefault().setActiveNode(null, node.getType());
                        }
                    }
                });
                projectByTree.put(db, project);
                treeByProject.put(project, db);
                notifyListeners(db, TreeEvent.TREE_OPEN);
            } else
                return false;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    private TreeDB createTreeDB(IProject project) {
        TreeDB db = null;
        ResourceStorage storage = new ResourceStorage(project);
        db = new TreeDB(storage);
        ResourcesPlugin.getWorkspace().addResourceChangeListener(
                storage.getResourceChangeListener());
        return db;
    }

    public boolean stopTracking(IProject project) {
        if (!isTracking(project)) {
            return false;
        }
        TreeDB db = getTree(project);
        ResourceStorage storage = (ResourceStorage) db.getStorage();
        ResourcesPlugin.getWorkspace().removeResourceChangeListener(
                storage.getResourceChangeListener());
        projectByTree.remove(db);
        treeByProject.remove(project);
        notifyListeners(db, TreeEvent.TREE_CLOSE);
        return true;
    }

    public boolean isTracking(IProject project) {
        boolean result = treeByProject.containsKey(project);
        return result;
    }

    public TreeDB[] getTrees() {
        return projectByTree.keySet().toArray(new TreeDB[0]);
    }

    public IProject[] getProjects() {
        return projectByTree.values().toArray(new IProject[0]);
    }
    
    private boolean isTrackable(IProject project) {
        try {
            return project.isOpen() && project.isNatureEnabled(ReqCatalogue.ID);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public void setProjects(IProject[] projects) {
        HashSet<IProject> current = new HashSet<IProject>(
                treeByProject.keySet());

        for (IProject project : projects) {

            try {
                if (isTrackable(project) && !isTracking(project)) {
                    trackProject(project);
                } else if (!isTrackable(project) && isTracking(project)) {
                    stopTracking(project);
                }
                current.remove(project);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        for (IProject iProject : current) {
            stopTracking(iProject);
        }
    }

    public boolean isInactive() {
        return inactivityCounter != 0;
    }

    public void turnOff() {
        inactivityCounter++;
    }

    public void turnOn() {
        if (inactivityCounter == 0) {
            throw new IllegalArgumentException("turnOn: inactivityCounter == 0");
        }
        inactivityCounter--;
    }

    public static TreeNode getNodeByResource(IFile file) {
        TreeDB db = getTracker().getTree(file.getProject());
        ResourceStorage rs = (ResourceStorage) db.getStorage();
        UUID uuid = rs.getUUId(file);
        if(uuid == null){
            return null;
        }
        return db.getNode(uuid);
    }

    public static IResource getResourceByNode(TreeNode node) {
        TreeDB db = node.getTreeDB();
        ResourceStorage rs = (ResourceStorage) db.getStorage();
        return rs.getResource(node.getUUId());
    }
}
