package com.unitesk.requality.eclipse.views;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.part.ViewPart;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.tools.DefaultTreeChangeListener;
import com.unitesk.requality.eclipse.tools.ITreeStateListener;
import com.unitesk.requality.eclipse.tools.IUIStateChangeListener;
import com.unitesk.requality.eclipse.tools.TreeDBNodeComparator;
import com.unitesk.requality.eclipse.tools.TreesTracker;

public abstract class TreesView extends ViewPart {

    protected TreeViewer tree;
    protected TreeNode curNode;

    protected IUIStateChangeListener uiChangeListener = new IUIStateChangeListener() {
        @Override
        public void newActiveNode(TreeNode node) {
            if (node != null 
                    && isTypeSupported(node.getType()) 
                    && !node.equals(curNode)) {
                select(node);
            }
        }
    };

    protected boolean isTypeSupported(String type) {
        return ((TreesContentProvider)tree.getContentProvider()).isTypeSupported(type);
    }
    
    protected ITreeChangeListener treeChangeListener = new DefaultTreeChangeListener() {
        @Override
        public void added(TreeNode node) {
            refresh(node.getParent());
        };
        @Override
        public void deleted(TreeNode node) {
            refresh(node.getParent());
        };
        @Override
        public void changed(TreeNode node) {
            refresh(node.getParent());
        }
        
        public void moved(TreeNode node, TreeNode oldParent) {
            refresh(oldParent);
            refresh(node.getParent());
        };
    };

    private ITreeStateListener treeStateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
            db.addListener(treeChangeListener);
            refresh(null);
        }

        @Override
        public void close(TreeDB db) {
            db.removeListener(treeChangeListener);
            refresh(null);
        }
    };

    protected ISelectionChangedListener createSelectionChangedListener() {
        ISelectionChangedListener treeSelection = new ISelectionChangedListener() {
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                if (event.getSelection() instanceof IStructuredSelection) {
                    Object obj = ((IStructuredSelection) event.getSelection())
                            .getFirstElement();
                    if (obj instanceof TreeNode) {
                        curNode = (TreeNode) obj;
                        Activator.getDefault().setActiveNode(curNode, curNode.getType());
                    }
                    if (obj instanceof TreeDB) {
                        for(String type: getTypes()) {
                            Activator.getDefault().setActiveNode(null, type);
                        }
                    }
                }
            }
        };
        return treeSelection;
    }

    abstract protected String[] getTypes();

    public void setFocus() {
        tree.getControl().setFocus();
    }

    public void createPartControl(Composite parent) {
        tree = new TreeViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.SINGLE);
        tree.setContentProvider(createContentProvider());
        tree.setLabelProvider(createLabelProvider());
        tree.setInput(ResourcesPlugin.getWorkspace().getRoot());
        tree.addSelectionChangedListener(createSelectionChangedListener());
        tree.setComparator(getViewerComparator());
        getSite().setSelectionProvider(tree);
        makeActions();
        hookContextMenu();
        hookDoubleClickAction();
        contributeToActionBars();

        // hook listeners
        for(TreeDB db : TreesTracker.getTracker().getTrees()) {
            db.addListener(treeChangeListener);
        }
        TreesTracker.getTracker().addListener(treeStateListener);
        Activator.getDefault().addChangeListener(uiChangeListener);
    }

    protected ViewerComparator getViewerComparator() {
        return new ViewerComparator(new TreeDBNodeComparator());
    }


    protected ITreeChangeListener getTreeChangeListener() {
        return treeChangeListener;
    }

    protected abstract IContentProvider createContentProvider();

    protected abstract ILabelProvider createLabelProvider();

    protected abstract void makeActions();

    protected abstract void hookContextMenu();

    protected abstract void hookDoubleClickAction();

    protected abstract void contributeToActionBars();

    public synchronized void refresh(final TreeNode node) {
        if (!tree.getControl().isDisposed()) {
            Display display = tree.getControl().getDisplay();
            if (!display.isDisposed()) {
                display.asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        if (node == null
                                || node.getQualifiedId().equals("/")
                                || node.getParent() ==null) {
                            tree.setInput(ResourcesPlugin.getWorkspace()
                                    .getRoot());
                            return;
                        }
                        tree.refresh(node);
                    }
                });
            }
        }
    }

    @Override
    public void dispose() {
        TreesTracker tracker = TreesTracker.getTracker();
        tracker.removeListener(treeStateListener);
        for(TreeDB db : tracker.getTrees()) {
            db.removeListener(treeChangeListener);
        }
        Activator.getDefault().removeChangeListener(uiChangeListener);
        super.dispose();
    }

    public void select(Object obj) {
        if (obj != null) {
            curNode = (TreeNode) obj;
            tree.expandToLevel(curNode.getParent(), 1);
            tree.setSelection(new StructuredSelection(obj), true);
        }
    }

    public ITreeSelection getSelection() {
        return (ITreeSelection) tree.getSelection();
    }

    public TreeNode getSelectedNode() {
        ITreeSelection ts = (ITreeSelection) tree.getSelection();
        if(ts == null) {
            return null;
        }
        return (TreeNode) ts.getFirstElement();
    }

}
